@@ -2,7 +2,7 @@
 
 import pytest
 import tinycss2
-from tinycss2.color3 import parse_color
+from tinycss2.color4 import parse_color
 
 from weasyprint.css import preprocess_declarations
 from weasyprint.css.properties import INITIAL_VALUES, ZERO_PIXELS
@@ -47,9 +47,17 @@ def test_empty_expander_value(expander):
         'text_decoration_line': {'blink', 'line-through', 'overline'},
     }),
     ('red', {'text_decoration_color': parse_color('red')}),
+    ('blue 1px', {
+        'text_decoration_color': parse_color('blue'),
+        'text_decoration_thickness': (1, 'px'),
+    }),
+    ('100% none', {
+        'text_decoration_line': 'none',
+        'text_decoration_thickness': (100, '%'),
+    }),
     ('inherit', {
         f'text_decoration_{key}': 'inherit'
-        for key in ('color', 'line', 'style')}),
+        for key in ('color', 'line', 'style', 'thickness')}),
 ))
 def test_text_decoration(rule, result):
     assert expand_to_dict(f'text-decoration: {rule}') == result
@@ -59,8 +67,8 @@ def test_text_decoration(rule, result):
 @pytest.mark.parametrize('rule', (
     'solid solid',
     'red red',
-    '1px',
     'underline none',
+    '1px 100%',
     'none none',
 ))
 def test_text_decoration_invalid(rule):
@@ -454,6 +462,74 @@ def test_border_image_invalid(rule, reason):
     assert_invalid(f'border-image: {rule}', reason)
 
 
+@assert_no_logs
+@pytest.mark.parametrize('rule, result', (
+    ('url(border.png) 27', {
+        'mask_border_source': ('url', 'https://weasyprint.org/foo/border.png'),
+        'mask_border_slice': ((27, None),),
+    }),
+    ('url(border.png) 10 / 4 / 2 round stretch', {
+        'mask_border_source': ('url', 'https://weasyprint.org/foo/border.png'),
+        'mask_border_slice': ((10, None),),
+        'mask_border_width': ((4, None),),
+        'mask_border_outset': ((2, None),),
+        'mask_border_repeat': (('round', 'stretch')),
+    }),
+    ('10 // 2', {
+        'mask_border_slice': ((10, None),),
+        'mask_border_outset': ((2, None),),
+    }),
+    ('5.5%', {
+        'mask_border_slice': ((5.5, '%'),),
+    }),
+    ('stretch 2 url("border.png")', {
+        'mask_border_source': ('url', 'https://weasyprint.org/foo/border.png'),
+        'mask_border_slice': ((2, None),),
+        'mask_border_repeat': (('stretch',)),
+    }),
+    ('1/2 round', {
+        'mask_border_slice': ((1, None),),
+        'mask_border_width': ((2, None),),
+        'mask_border_repeat': (('round',)),
+    }),
+    ('none', {
+        'mask_border_source': ('none', None),
+    }),
+    ('url(border.png) 27 alpha', {
+        'mask_border_source': ('url', 'https://weasyprint.org/foo/border.png'),
+        'mask_border_slice': ((27, None),),
+        'mask_border_mode': 'alpha',
+    }),
+    ('url(border.png) 27 luminance', {
+        'mask_border_source': ('url', 'https://weasyprint.org/foo/border.png'),
+        'mask_border_slice': ((27, None),),
+        'mask_border_mode': 'luminance',
+    }),
+))
+def test_mask_border(rule, result):
+    assert expand_to_dict(f'mask-border: {rule}') == result
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, reason', (
+    ('url(border.png) url(border.png)', 'multiple source'),
+    ('10 10 10 10 10', 'multiple slice'),
+    ('1 / 2 / 3 / 4', 'invalid'),
+    ('/1', 'invalid'),
+    ('/1', 'invalid'),
+    ('round round round', 'invalid'),
+    ('-1', 'invalid'),
+    ('1 repeat 2', 'multiple slice'),
+    ('1% // 1%', 'invalid'),
+    ('1 / repeat', 'invalid'),
+    ('', 'no value'),
+    ('alpha alpha', 'multiple mode'),
+    ('alpha luminance', 'multiple mode'),
+))
+def test_mask_border_invalid(rule, reason):
+    assert_invalid(f'mask-border: {rule}', reason)
+
+
 @assert_no_logs
 @pytest.mark.parametrize('rule, result', (
     ('12px My Fancy Font, serif', {
