@@ -1,5 +1,6 @@
 """Test validation of properties."""
 
+import logging
 from math import pi
 
 import pytest
@@ -12,10 +13,10 @@ from weasyprint.images import LinearGradient, RadialGradient
 from ..testing_utils import assert_no_logs, capture_logs
 
 
-def get_value(css, expected_error=None):
+def get_value(css, expected_error=None, level=None):
     declarations = tinycss2.parse_blocks_contents(css)
 
-    with capture_logs() as logs:
+    with capture_logs(level=level) as logs:
         base_url = 'https://weasyprint.org/foo/'
         declarations = list(preprocess_declarations(base_url, declarations))
 
@@ -36,13 +37,13 @@ def get_default_value(values, index, default):
     return values[index] or default
 
 
-def assert_invalid(css, message='invalid'):
-    assert get_value(css, message) is None
+def assert_invalid(css, message='invalid', level=None):
+    assert get_value(css, message, level) is None
 
 
 @assert_no_logs
 def test_not_print():
-    assert_invalid('volume: 42', 'does not apply for the print media')
+    assert_invalid('volume: 42', 'does not apply for the print media', logging.DEBUG)
 
 
 @assert_no_logs
@@ -60,7 +61,10 @@ def test_normal_prefix():
 
 @assert_no_logs
 def test_unknown_prefix():
-    assert_invalid('-unknown-display: block', 'prefixed selectors are ignored')
+    assert_invalid(
+        '-unknown-display: block',
+        'prefixed selectors are ignored',
+        logging.DEBUG)
 
 
 @assert_no_logs
@@ -470,6 +474,135 @@ def test_border_image_repeat_invalid(rule):
     assert_invalid(f'border-image-repeat: {rule}')
 
 
+@assert_no_logs
+@pytest.mark.parametrize('rule, value', (
+    ('1', ((1, None),)),
+    ('1 2    3 4', ((1, None), (2, None), (3, None), (4, None))),
+    ('50% 1000.1 0', ((50, '%'), (1000.1, None), (0, None))),
+    ('1% 2% 3% 4%', ((1, '%'), (2, '%'), (3, '%'), (4, '%'))),
+    ('fill 10% 20', ('fill', (10, '%'), (20, None))),
+    ('0 1 0.5 fill', ((0, None), (1, None), (0.5, None), 'fill')),
+))
+def test_mask_border_slice(rule, value):
+    assert get_value(f'mask-border-slice: {rule}') == value
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule', (
+    'none',
+    '1, 2',
+    '-10',
+    '-10%',
+    '1 2 3 -10%',
+    '-0.3',
+    '1 fill 2',
+    'fill 1 2 3 fill',
+))
+def test_mask_border_slice_invalid(rule):
+    assert_invalid(f'mask-border-slice: {rule}')
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, value', (
+    ('1', ((1, None),)),
+    ('1 2    3 4', ((1, None), (2, None), (3, None), (4, None))),
+    ('50% 1000.1 0', ((50, '%'), (1000.1, None), (0, None))),
+    ('1% 2px 3em 4', ((1, '%'), (2, 'px'), (3, 'em'), (4, None))),
+    ('auto', ('auto',)),
+    ('1 auto', ((1, None), 'auto')),
+    ('auto auto', ('auto', 'auto')),
+    ('auto auto auto 2', ('auto', 'auto', 'auto', (2, None))),
+))
+def test_mask_border_width(rule, value):
+    assert get_value(f'mask-border-width: {rule}') == value
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule', (
+    'none',
+    '1, 2',
+    '1 -2',
+    '-10',
+    '-10%',
+    '1px 2px 3px -10%',
+    '-3px',
+    'auto auto auto auto auto',
+    '1 2 3 4 5',
+))
+def test_mask_border_width_invalid(rule):
+    assert_invalid(f'mask-border-width: {rule}')
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, value', (
+    ('1', ((1, None),)),
+    ('1 2    3 4', ((1, None), (2, None), (3, None), (4, None))),
+    ('50px 1000.1 0', ((50, 'px'), (1000.1, None), (0, None))),
+    ('1in 2px 3em 4', ((1, 'in'), (2, 'px'), (3, 'em'), (4, None))),
+))
+def test_mask_border_outset(rule, value):
+    assert get_value(f'mask-border-outset: {rule}') == value
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule', (
+    'none',
+    'auto',
+    '1, 2',
+    '-10',
+    '1 -2',
+    '10%',
+    '1px 2px 3px -10px',
+    '-3px',
+    '1 2 3 4 5',
+))
+def test_mask_border_outset_invalid(rule):
+    assert_invalid(f'mask-border-outset: {rule}')
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, value', (
+    ('stretch', ('stretch',)),
+    ('repeat repeat', ('repeat', 'repeat')),
+    ('round     space', ('round', 'space')),
+))
+def test_mask_border_repeat(rule, value):
+    assert get_value(f'mask-border-repeat: {rule}') == value
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule', (
+    'none',
+    'test',
+    'round round round',
+    'stretch space round',
+    'repeat test',
+))
+def test_mask_border_repeat_invalid(rule):
+    assert_invalid(f'mask-border-repeat: {rule}')
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, value', (
+    ('alpha', 'alpha'),
+    ('luminance', 'luminance'),
+    ('alpha     ', 'alpha'),
+))
+def test_mask_border_mode(rule, value):
+    assert get_value(f'mask-border-mode: {rule}') == value
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule', (
+    'none',
+    'test',
+    'alpha alpha',
+    'alpha luminance',
+))
+def test_mask_border_mode_invalid(rule):
+    assert_invalid(f'mask-border-mode: {rule}')
+
+
 @assert_no_logs
 @pytest.mark.parametrize('rule, value', (
     ('test content(text)', (('test', (('content()', 'text'),)),)),
@@ -808,6 +941,8 @@ def test_grid_template_columns_rows_invalid(rule):
      (('title', 'board'), ('stats', 'board'))),
     ('". a" "b a" ".a"',
      ((None, 'a'), ('b', 'a'), (None, 'a'))),
+    ('"a b b" "c b b" "d e f"',
+     (('a', 'b', 'b'), ('c', 'b', 'b'), ('d', 'e', 'f'))),
 ))
 def test_grid_template_areas(rule, value):
     assert get_value(f'grid-template-areas: {rule}') == value
@@ -829,12 +964,12 @@ def test_grid_template_areas_invalid(rule):
 @pytest.mark.parametrize('rule, value', (
     ('auto', 'auto'),
     ('4', (None, 4, None)),
-    ('C', (None, None, 'c')),
+    ('C', (None, None, 'C')),
     ('4 c', (None, 4, 'c')),
     ('col -4', (None, -4, 'col')),
     ('span c 4', ('span', 4, 'c')),
     ('span 4 c', ('span', 4, 'c')),
-    ('4 span c', ('span', 4, 'c')),
+    ('4 span C', ('span', 4, 'C')),
     ('super 4 span', ('span', 4, 'super')),
 ))
 def test_grid_line(rule, value):
