@@ -129,7 +129,7 @@ def test_variable_chain():
 
 @assert_no_logs
 def test_variable_chain_root():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/1656
+    # Regression test for #1656.
     page, = render_pages('''
       <style>
         html { --var2: 10px; --var1: var(--var2); width: var(--var1) }
@@ -150,13 +150,13 @@ def test_variable_self():
 def test_variable_loop():
     page, = render_pages('''
       <style>
-        html { --var1: var(--var2); --var2: var(--var1) }
+        html { --var1: var(--var2); --var2: var(--var1); padding: var(--var1) }
       </style>
     ''')
 
 
 def test_variable_chain_root_missing():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/1656
+    # Regression test for #1656.
     page, = render_pages('''
       <style>
         html { --var1: var(--var-missing); width: var(--var1) }
@@ -165,7 +165,7 @@ def test_variable_chain_root_missing():
 
 
 def test_variable_chain_root_missing_inherited():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/2164
+    # Regression test for #2164.
     page, = render_pages('''
       <style>
         html { --var1: var(--var-missing); font: var(--var1) }
@@ -342,7 +342,7 @@ def test_variable_shorthand_background_invalid(var, background):
 
 @assert_no_logs
 def test_variable_initial():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/2075
+    # Regression test for #2075.
     page, = render_pages('''
       <style>
         html { --var: initial }
@@ -358,7 +358,7 @@ def test_variable_initial():
 
 @assert_no_logs
 def test_variable_initial_default():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/2075
+    # Regression test for #2075.
     page, = render_pages('''
       <style>
         p { --var: initial; width: var(--var, 10px) }
@@ -373,7 +373,7 @@ def test_variable_initial_default():
 
 @assert_no_logs
 def test_variable_initial_default_var():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/2075
+    # Regression test for #2075.
     page, = render_pages('''
       <style>
         p { --var: initial; width: var(--var, var(--var)) }
@@ -401,7 +401,7 @@ def test_variable_fallback(prop):
 
 @assert_no_logs
 def test_variable_list_content():
-    # Regression test for https://github.com/Kozea/WeasyPrint/issues/1287
+    # Regression test for #1287.
     page, = render_pages('''
       <style>
         :root { --var: "Page " counter(page) "/" counter(pages) }
