@@ -965,6 +965,29 @@ def test_flex_item_overflow():
     assert text2.text == 'e'
 
 
+@assert_no_logs
+@pytest.mark.parametrize('direction', ('row', 'column'))
+def test_flex_item_child_bottom_margin(direction):
+    # Regression test for issue #2449.
+    page, = render_pages('''
+      <div style="display: flex; font: 2px weasyprint; flex-direction: %s">
+        <section>
+          <div style="margin: 2px 0">ab</div>
+        </section>
+      </div>''' % direction)
+    html, = page.children
+    body, = html.children
+    flex, = body.children
+    assert flex.content_box_y() == 0
+    assert flex.height == 6
+    section, = flex.children
+    assert section.content_box_y() == 0
+    assert section.height == 6
+    div, = section.children
+    assert div.content_box_y() == 2
+    assert div.height == 2
+
+
 @assert_no_logs
 def test_flex_direction_row_inline_block():
     # Regression test for issue #1652.
@@ -1110,8 +1133,6 @@ def test_flex_item_auto_margin_main():
 
 @assert_no_logs
 def test_flex_item_auto_margin_cross():
-    # TODO: we keep this test because it used to crash, but we have to fix the top
-    # margin.
     page, = render_pages('''
       <article style="display: flex; height: 100px">
         <div style="margin-top: auto; height: 10px; width: 10px"></div>
@@ -1123,7 +1144,7 @@ def test_flex_item_auto_margin_cross():
     div, = article.children
     assert div.height == 10
     assert div.width == 10
-    # assert div.margin_top == 90
+    assert div.margin_top == 90
 
 
 @assert_no_logs
@@ -1831,3 +1852,80 @@ def test_flex_item_replaced():
     div, = body.children
     svg, = div.children
     assert svg.width == svg.height == 100
+
+
+@assert_no_logs
+def test_flex_nested_column():
+    # Regression test for issue #2442.
+    page, = render_pages('''
+      <section style="display: flex; flex-direction: column; width: 200px">
+        <div style="display: flex; flex-direction: column">
+          <p>
+            A
+          </p>
+        </div>
+      </section>
+    ''')
+    html, = page.children
+    body, = html.children
+    section, = body.children
+    div, = section.children
+    p, = div.children
+    assert p.width == 200
+
+
+@assert_no_logs
+def test_flex_blockify_image():
+    page, = render_pages('''
+      <article style="display: flex; line-height: 2">
+        <img src="pattern.png">
+      </article>
+    ''')
+    html, = page.children
+    body, = html.children
+    article, = body.children
+    img, = article.children
+    assert article.height == img.height == 4
+
+
+@assert_no_logs
+def test_flex_image_max_width():
+    page, = render_pages('''
+      <article style="display: flex">
+        <img src="pattern.png" style="max-width: 2px">
+      </article>
+    ''')
+    html, = page.children
+    body, = html.children
+    article, = body.children
+    img, = article.children
+    assert article.height == img.height == img.width == 2
+
+
+@assert_no_logs
+def test_flex_image_max_height():
+    page, = render_pages('''
+      <article style="display: flex">
+        <img src="pattern.png" style="max-height: 2px">
+      </article>
+    ''')
+    html, = page.children
+    body, = html.children
+    article, = body.children
+    img, = article.children
+    assert article.height == img.height == img.width == 2
+
+
+@assert_no_logs
+def test_flex_image_min_width():
+    page, = render_pages('''
+      <article style="display: flex; width: 20px">
+        <img style="min-width: 10px; flex: 1 0 auto" src="pattern.png">
+        <div style="flex: 1 0 1px"></div>
+      </article>
+    ''')
+    html, = page.children
+    body, = html.children
+    article, = body.children
+    img, div = article.children
+    assert article.height == img.height == img.width == div.height == 10
